#include <stdio.h>
#include <sys/types.h>
#include <math.h>

#include "../include/os.h"

#ifdef __MSW__
# include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>

#include "gw.h"

#include "sarreality.h"
#include "obj.h"
#include "sfm.h"
#include "sar.h"
#include "sardrawpm.h"


static void SARDrawPremodeledRadioTowerRadioRight(double tz);
static void SARDrawPremodeledRadioTowerRadioLeft(double tz);
void SARDrawPremodeledRadioTower(
        gw_display_struct *display,
        sar_scene_struct *scene,
        sar_object_struct *obj_ptr,
        sar_object_premodeled_struct *obj_premodeled_ptr,
        double camera_dist,
        Boolean draw_for_gcc
);


#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))


/*
 *      Draws right facing radar dish.
 */
static void SARDrawPremodeledRadioTowerRadioRight(double tz)
{
        glBegin(GL_QUADS);
        {
            glColor4d(1.0, 1.0, 1.0, 1.0);

            glNormal3d(0.0, 1.0, 0.0);

            glVertex3d(3.0,  0.0 + tz, -( 0.0));
            glVertex3d(6.0,  0.0 + tz, -( 8.0));
            glVertex3d(6.0, -5.6 + tz, -( 5.6));
            glVertex3d(6.0, -8.0 + tz, -( 0.0));

            glVertex3d(6.0, -8.0 + tz, -( 0.0));
            glVertex3d(6.0, -5.6 + tz, -(-5.6));
            glVertex3d(6.0,  0.0 + tz, -(-8.0));
            glVertex3d(3.0,  0.0 + tz, -( 0.0));

            glVertex3d(6.0,  8.0 + tz, -( 0.0));
            glVertex3d(6.0,  5.6 + tz, -( 5.6));
            glVertex3d(6.0,  0.0 + tz, -( 8.0));
            glVertex3d(3.0,  0.0 + tz, -( 0.0));

            glVertex3d(3.0,  0.0 + tz, -( 0.0));
            glVertex3d(6.0,  0.0 + tz, -(-8.0));
            glVertex3d(6.0,  5.6 + tz, -(-5.6));
            glVertex3d(6.0,  8.0 + tz, -( 0.0));

            glColor4d(0.3, 0.3, 0.3, 1.0);

            glVertex3d(6.0, -8.0 + tz, -( 0.0));
            glVertex3d(6.0, -5.6 + tz, -( 5.6));
            glVertex3d(6.0,  0.0 + tz, -( 8.0));
            glVertex3d(6.0,  0.0 + tz, -( 0.0));

            glVertex3d(6.0,  0.0 + tz, -( 0.0));
            glVertex3d(6.0,  0.0 + tz, -(-8.0));
            glVertex3d(6.0, -5.6 + tz, -(-5.6));
            glVertex3d(6.0, -8.0 + tz, -( 0.0));

            glVertex3d(6.0,  0.0 + tz, -( 0.0));
            glVertex3d(6.0,  0.0 + tz, -( 8.0));
            glVertex3d(6.0,  5.6 + tz, -( 5.6));
            glVertex3d(6.0,  8.0 + tz, -( 0.0));

            glVertex3d(6.0,  8.0 + tz, -( 0.0));
            glVertex3d(6.0,  5.6 + tz, -(-5.6));
            glVertex3d(6.0,  0.0 + tz, -(-8.0));
            glVertex3d(6.0,  0.0 + tz, -( 0.0));
        }
        glEnd();
}

/*
 *	Draws left facing radar dish.
 */
static void SARDrawPremodeledRadioTowerRadioLeft(double tz)
{
	glBegin(GL_QUADS);
	{
	    glColor4d(1.0, 1.0, 1.0, 1.0);

	    glNormal3d(0.0, 1.0, 0.0);

            glVertex3d(-6.0, -8.0 + tz, -( 0.0));
            glVertex3d(-6.0, -5.6 + tz, -( 5.6));
            glVertex3d(-6.0,  0.0 + tz, -( 8.0));
	    glVertex3d(-3.0,  0.0 + tz, -( 0.0));

            glVertex3d(-3.0,  0.0 + tz, -( 0.0));
            glVertex3d(-6.0,  0.0 + tz, -(-8.0));
            glVertex3d(-6.0, -5.6 + tz, -(-5.6));
            glVertex3d(-6.0, -8.0 + tz, -( 0.0));

            glVertex3d(-3.0,  0.0 + tz, -( 0.0));
            glVertex3d(-6.0,  0.0 + tz, -( 8.0));
            glVertex3d(-6.0,  5.6 + tz, -( 5.6));
            glVertex3d(-6.0,  8.0 + tz, -( 0.0));

            glVertex3d(-6.0,  8.0 + tz, -( 0.0));
            glVertex3d(-6.0,  5.6 + tz, -(-5.6));
            glVertex3d(-6.0,  0.0 + tz, -(-8.0));
            glVertex3d(-3.0,  0.0 + tz, -( 0.0));

	    glColor4d(0.3, 0.3, 0.3, 1.0);

            glVertex3d(-6.0,  0.0 + tz, -( 0.0));
            glVertex3d(-6.0,  0.0 + tz, -( 8.0));
            glVertex3d(-6.0, -5.6 + tz, -( 5.6));
            glVertex3d(-6.0, -8.0 + tz, -( 0.0));

            glVertex3d(-6.0, -8.0 + tz, -( 0.0));
            glVertex3d(-6.0, -5.6 + tz, -(-5.6));
            glVertex3d(-6.0,  0.0 + tz, -(-8.0));
            glVertex3d(-6.0,  0.0 + tz, -( 0.0));

            glVertex3d(-6.0,  8.0 + tz, -( 0.0));
            glVertex3d(-6.0,  5.6 + tz, -( 5.6));
            glVertex3d(-6.0,  0.0 + tz, -( 8.0));
            glVertex3d(-6.0,  0.0 + tz, -( 0.0));

            glVertex3d(-6.0,  0.0 + tz, -( 0.0));
            glVertex3d(-6.0,  0.0 + tz, -(-8.0));
            glVertex3d(-6.0,  5.6 + tz, -(-5.6));
            glVertex3d(-6.0,  8.0 + tz, -( 0.0));
	}
	glEnd();
}

/*
 *      Draws a premodeled tower.
 */
void SARDrawPremodeledRadioTower(
        gw_display_struct *display,
        sar_scene_struct *scene,
        sar_object_struct *obj_ptr,
        sar_object_premodeled_struct *obj_premodeled_ptr,
        double camera_dist,
        Boolean draw_for_gcc
)
{
        GLenum shade_model_mode;
        double sx, sy, sz;              /* Scale. */


        /* All hard coded coordinates will produce an object of 100 unit
         * size (height = 100.0). So the sx, sy, and sz scale coefficients
         * will simply multiply each point / 100.0.
         */

        /* Calculate scales, all based on height. */
	sx = sy = sz = obj_premodeled_ptr->height / 100.0;

        /* Set flat shade model state. */
        shade_model_mode = display->state_gl.shade_model_mode;
        StateGLShadeModel(&display->state_gl, GL_FLAT);

        glColor4d(0.45, 0.45, 0.45, 1.0);

        /* Draw close range model? */
        if(camera_dist < 2000.0)
        {
	    /* Guy wires. */
	    glBegin(GL_LINES);
	    {
		glNormal3d(0.0, 1.0, 0.0);

		glVertex3d(-29.0 * sx,  0.0 * sz, -(-29.0 * sy));
                glVertex3d(  0.0 * sx, 75.0 * sz, -(  0.0 * sy));

                glVertex3d( 29.0 * sx,  0.0 * sz, -(-29.0 * sy));
                glVertex3d(  0.0 * sx, 75.0 * sz, -(  0.0 * sy));

                glVertex3d(  0.0 * sx,  0.0 * sz, -( 40.0 * sy));
                glVertex3d(  0.0 * sx, 75.0 * sz, -(  0.0 * sy));
	    }
	    glEnd();

	    sx = sy = 1.0;
	    glBegin(GL_LINE_STRIP);
	    {
		glNormal3d(0.0, 1.0, 0.0);

                glVertex3d(  3.0 * sx,   0.0 * sz, -( 0.0 * sy));
                glVertex3d( -3.0 * sx,   0.0 * sz, -( 0.0 * sy));
                glVertex3d( -3.0 * sx, 100.0 * sz, -( 0.0 * sy));
                glVertex3d(  3.0 * sx,  90.0 * sz, -( 0.0 * sy));
                glVertex3d( -3.0 * sx,  80.0 * sz, -( 0.0 * sy));
                glVertex3d(  3.0 * sx,  70.0 * sz, -( 0.0 * sy));
                glVertex3d( -3.0 * sx,  60.0 * sz, -( 0.0 * sy));
                glVertex3d(  3.0 * sx,  50.0 * sz, -( 0.0 * sy));
                glVertex3d( -3.0 * sx,  40.0 * sz, -( 0.0 * sy));
                glVertex3d(  3.0 * sx,  30.0 * sz, -( 0.0 * sy));
                glVertex3d( -3.0 * sx,  20.0 * sz, -( 0.0 * sy));
                glVertex3d(  3.0 * sx,  10.0 * sz, -( 0.0 * sy));
                glVertex3d( -3.0 * sx,   0.0 * sz, -( 0.0 * sy));

                glVertex3d( -3.0 * sx,   0.0 * sz, -(-0.2 * sy));
                glVertex3d(  0.0 * sx,   0.0 * sz, -( 5.0 * sy));
                glVertex3d(  0.0 * sx, 100.0 * sz, -( 5.0 * sy));
                glVertex3d( -3.0 * sx,  90.0 * sz, -(-0.2 * sy));
                glVertex3d(  0.0 * sx,  80.0 * sz, -( 5.0 * sy));
                glVertex3d( -3.0 * sx,  70.0 * sz, -(-0.2 * sy));
                glVertex3d(  0.0 * sx,  60.0 * sz, -( 5.0 * sy));
                glVertex3d( -3.0 * sx,  50.0 * sz, -(-0.2 * sy));
                glVertex3d(  0.0 * sx,  40.0 * sz, -( 5.0 * sy));
                glVertex3d( -3.0 * sx,  30.0 * sz, -(-0.2 * sy));
                glVertex3d(  0.0 * sx,  20.0 * sz, -( 5.0 * sy));
                glVertex3d( -3.0 * sx,  10.0 * sz, -(-0.2 * sy));
                glVertex3d(  0.0 * sx,   0.0 * sz, -( 5.0 * sy));

                glVertex3d(  0.0 * sx,   0.0 * sz, -( 5.2 * sy));
                glVertex3d(  3.0 * sx,   0.0 * sz, -( 0.0 * sy));
                glVertex3d(  3.0 * sx, 100.0 * sz, -( 0.0 * sy));
                glVertex3d(  0.0 * sx,  90.0 * sz, -( 5.2 * sy));
                glVertex3d(  3.0 * sx,  80.0 * sz, -( 0.0 * sy));
                glVertex3d(  0.0 * sx,  70.0 * sz, -( 5.2 * sy));
                glVertex3d(  3.0 * sx,  60.0 * sz, -( 0.0 * sy));
                glVertex3d(  0.0 * sx,  50.0 * sz, -( 5.2 * sy));
                glVertex3d(  3.0 * sx,  40.0 * sz, -( 0.0 * sy));
                glVertex3d(  0.0 * sx,  30.0 * sz, -( 5.2 * sy));
                glVertex3d(  3.0 * sx,  20.0 * sz, -( 0.0 * sy));
                glVertex3d(  0.0 * sx,  10.0 * sz, -( 5.2 * sy));
                glVertex3d(  3.0 * sx,   0.0 * sz, -( 0.0 * sy));
	    }
	    glEnd();

	    SARDrawPremodeledRadioTowerRadioLeft(50.0 * sz);
            SARDrawPremodeledRadioTowerRadioRight(75.0 * sz);
	}
        else
        {
            /* Draw far model. */
            glBegin(GL_LINES);
            {
                glNormal3d(0.0, 1.0, 0.0);

                glVertex3d(0.0 * sx,   0.0 * sz, -(0.0 * sy));
                glVertex3d(0.0 * sx, 100.0 * sz, -(0.0 * sy));
            }
            glEnd();
        }

        /* Restore shade model. */
        StateGLShadeModel(&display->state_gl, shade_model_mode);
}
