#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/stat.h>

char *DoLoadFile(char *path, int *w, int *h)
{
	int n, x, y, buf_len;
	FILE *fp;
	off_t file_size;
	struct stat stat_buf;
	char *buf;
	char *strptr;


	if(stat(path, &stat_buf))
	    return(NULL);

	file_size = stat_buf.st_size;
	if(file_size <= 0)
	    return(NULL);

	/* Open file. */
	fp = fopen(path, "r");
	if(fp == NULL)
	    return(NULL);

	buf = (char *)malloc((file_size + 1) * sizeof(char));
	if(buf == NULL)
	{
	    fclose(fp);
	    return(NULL);
	}
        fread(buf, file_size, sizeof(char), fp);
        buf[file_size] = '\0';  /* Null terminate last byte. */
	fclose(fp);

        /* Calculate `longest' line width_bytes. */
	n = 0;
	*w = 0; *h = 0;
	strptr = buf;
        while(*strptr != '\0')
        {
            if((*strptr == '\n') ||
               (*strptr == '\r')
            )
            {
                *h = *h + 1;

                if(*w < n)
		    *w = n;

                n = 0;
                strptr++;

                continue;
            }

            n++;
            strptr++;
        }
	free(buf);
	if((*w < 1) ||
           (*h < 1)
	)
	    return(NULL);


	/* Open file again for reading as a map. */
        fp = fopen(path, "r");
        if(fp == NULL)
            return(NULL);

	buf_len = (*w) * (*h) * sizeof(char);
        buf = (char *)malloc(buf_len + 1);
        if(buf == NULL)
	{
	    fclose(fp);
	    return(NULL);
	}

	/* Clear buffer and set last byte to null. */
	memset(buf, ' ', buf_len);
	buf[buf_len] = '\0';	/* Yes this byte is allocated. */

	for(y = 0; y < (*h); y++)
	{
	    x = 0;
	    while(1)
	    {
		n = fgetc(fp);
		if((n == '\n') ||
                   (n == '\r') ||
		   (n == EOF)
		)
		    break;

		buf[(y * (*w)) + x] = n;

		x++;
	    }
	}

	fclose(fp);

	return(buf);
}


u_int8_t *DoCreateBitmap(char *buf, int w, int h)
{
	int i, n, x, y;
	u_int8_t *bm_buf;
	int bm_width_bytes;


	if(buf == NULL)
	    return(NULL);

	/* Calculate bitmap width. */
	if((w % 8) == 0)
	    bm_width_bytes = w / 8;
	else
	    bm_width_bytes = (w / 8) + 1;
	if(bm_width_bytes < 1)
	    bm_width_bytes = 1;


	/* Allocate bitmap buffer. */
	bm_buf = (u_int8_t *)calloc(
	    bm_width_bytes * h,
	    sizeof(u_int8_t)
	);
	if(bm_buf == NULL)
	    return(NULL);

	/* Go through each line. */
	for(y = 0; y < h; y++)
	{
	    for(x = 0; x < w; x++)
	    {
		if(buf[(y * w) + x] == ' ')
		    continue;

		i = x / 8;
		n = 7 - (x % 8);
		bm_buf[(y * bm_width_bytes) + i] |= (1 << n);
	    }
	}

	return(bm_buf);
}


int main(int argc, char **argv)
{
	int x, y, bm_w, bm_h;
	char *file_buf;
	u_int8_t *bitmap_buf;


	if(argc < 2)
	{
	    printf("Usage: texttobitmap <infile> > <outfile>\n");
	    return(1);
	}


	file_buf = DoLoadFile(argv[1], &bm_w, &bm_h);
	if(file_buf == NULL)
	{
	    fprintf(stderr, "%s: Cannot load file.\n", argv[1]);
	    return(1);
	}

	bitmap_buf = DoCreateBitmap(file_buf, bm_w, bm_h);
	free(file_buf);
	if(bitmap_buf == NULL)
	{
	    fprintf(stderr, "Could not generate bitmap.\n");
	    return(1);
	}

	if((bm_w % 8) == 0)
	    bm_w = bm_w / 8;
	else
	    bm_w = (bm_w / 8) + 1;
	for(y = bm_h - 1; y >= 0; y--)
	{
	    for(x = 0; x < bm_w; x++)
	    {
	        printf("0x%.2x, ", bitmap_buf[(y * bm_w) + x]);
	    }
	    printf("\n");
	}
	free(bitmap_buf);

	return(0);
}
